package com.prowidesoftware.brilliance.mx2mt.mt103.impl;


import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.DateUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation11;
import com.prowidesoftware.swift.model.mx.dic.FIToFICustomerCreditTransferV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.datatype.XMLGregorianCalendar;
import java.util.ArrayList;
import java.util.List;

public class Field13CGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(Field13CGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field13CGenerate start.");
        try {
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            FIToFICustomerCreditTransferV02 fiToFICstmrCdtTrf = obj.getFIToFICstmrCdtTrf();
            List<CreditTransferTransactionInformation11> cdtTrfTxInf = fiToFICstmrCdtTrf.getCdtTrfTxInf();
            List<String> filed13cList = new ArrayList<>();
            if (cdtTrfTxInf != null && cdtTrfTxInf.size() > 0){
                if (cdtTrfTxInf.get(0) != null
                    && cdtTrfTxInf.get(0).getSttlmTmIndctn() != null){
                    if (cdtTrfTxInf.get(0).getSttlmTmIndctn().getDbtDtTm() != null){
                        XMLGregorianCalendar dbtDtTm = cdtTrfTxInf.get(0).getSttlmTmIndctn().getDbtDtTm();
                        String hmm = DateUtil.format(dbtDtTm, "HHmm");
                        filed13cList.add("/SNDTIME/"+hmm+"+0800");
                    }
                    if (cdtTrfTxInf.get(0).getSttlmTmIndctn().getCdtDtTm() != null){
                        XMLGregorianCalendar dbtDtTm = cdtTrfTxInf.get(0).getSttlmTmIndctn().getCdtDtTm();
                        String hmm = DateUtil.format(dbtDtTm, "HHmm");
                        filed13cList.add("/RNCTIME/"+hmm+"+0800");
                    }
                    if (cdtTrfTxInf.get(0).getSttlmTmReq().getCLSTm() != null){
                        XMLGregorianCalendar dbtDtTm = cdtTrfTxInf.get(0).getSttlmTmReq().getCLSTm();
                        String hmm = DateUtil.format(dbtDtTm, "HHmm");
                        filed13cList.add("/CLSTIME/"+hmm+"+0800");
                    }
                }
            }
            if (filed13cList.size() > 0) {
                filed13cList.forEach(filed13c -> logger.info(":13c:"+filed13c));
                map.put("filed13CList", filed13cList);
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            e.printStackTrace();
            throw new SwiftException("ERROR", "Convert MT103 Field13C error");
        }

    }
}
