package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation11;
import com.prowidesoftware.swift.model.mx.dic.FIToFICustomerCreditTransferV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 若<Cd>节点值为“SDVA”，则取默认值“CRED”，否则直接取<Prtry>节点作为23B的值
 * <CdtTrfTxInf>[1]<PmtTpInf><SvcLvl><Cd>
 * <CdtTrfTxInf>[1]<PmtTpInf><SvcLvl><Prtry>
 */
public class Field23BGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(Field23BGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {

        logger.info("Field23BGenerate start.");
        try {
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            FIToFICustomerCreditTransferV02 fiToFICstmrCdtTrf = obj.getFIToFICstmrCdtTrf();
            List<CreditTransferTransactionInformation11> cdtTrfTxInf = fiToFICstmrCdtTrf.getCdtTrfTxInf();
            if (cdtTrfTxInf != null && cdtTrfTxInf.size() > 0
                    && cdtTrfTxInf.get(0) != null
                    && cdtTrfTxInf.get(0).getPmtTpInf() != null
                    && cdtTrfTxInf.get(0).getPmtTpInf().getSvcLvl() != null
                    && cdtTrfTxInf.get(0).getPmtTpInf().getSvcLvl().getCd() != null) {
                String field23B_cd = cdtTrfTxInf.get(0).getPmtTpInf().getSvcLvl().getCd();
                if ("SDVA".equals(field23B_cd)) {
                    map.put("field23B","CRED");
                } else if (cdtTrfTxInf.get(0).getPmtTpInf().getSvcLvl().getPrtry() != null){
                    String prtry = cdtTrfTxInf.get(0).getPmtTpInf().getSvcLvl().getPrtry();
                    map.put("field23B",prtry);
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert  MT103 Field23B error");
        }
    }
}
