package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation11;
import com.prowidesoftware.swift.model.mx.dic.FIToFICustomerCreditTransferV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * //1）code部分的取值：
 *         //1.1）若<PmtTpInf><SvcLvl><Cd>节点为“SDVA”，则取值“SDVA”；
 *         //1.2）若<CtgyPurp><Cd>节点有值，则直接取此节点作为23E的值；
 *         //1.3）若<InstrForCdtrAgt><Cd>有值，则直接取此节点作为23E的值
 *         //2）直接取节点<InstrInf>的内容作为addtion information的值
 *         <CdtTrfTxInf>[1]<PmtTpInf><CtgyPurp><Cd>
 * <CdtTrfTxInf>[1]<InstrForCdtrAgt><Cd>
 * <CdtTrfTxInf>[1]<InstrForCdtrAgt><InstrInf>
 */
public class Field23EGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(Field23EGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field23EGenerate start.");
        try {
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            FIToFICustomerCreditTransferV02 fiToFICstmrCdtTrf = obj.getFIToFICstmrCdtTrf();
            List<CreditTransferTransactionInformation11> cdtTrfTxInf = fiToFICstmrCdtTrf.getCdtTrfTxInf();
            List<String> field23EList = new ArrayList<String>();
            if (cdtTrfTxInf != null && cdtTrfTxInf.size() > 0){
                if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getPmtTpInf() != null
                        && cdtTrfTxInf.get(0).getPmtTpInf().getSvcLvl() != null
                        && cdtTrfTxInf.get(0).getPmtTpInf().getSvcLvl().getCd() != null){
                    String field23E_cd = cdtTrfTxInf.get(0).getPmtTpInf().getSvcLvl().getCd();
                    if ("SDVA".equals(field23E_cd)) {
                        field23EList.add("SDVA");
                    }
                    if (cdtTrfTxInf.get(0).getPmtTpInf().getCtgyPurp() !=null
                            && cdtTrfTxInf.get(0).getPmtTpInf().getCtgyPurp().getCd() != null){
                        String field23E = cdtTrfTxInf.get(0).getPmtTpInf().getCtgyPurp().getCd();
                        field23EList.add(field23E);
                    }
                    if (cdtTrfTxInf.get(0).getInstrForCdtrAgt() != null
                            && cdtTrfTxInf.get(0).getInstrForCdtrAgt().size() > 0
                            && cdtTrfTxInf.get(0).getInstrForCdtrAgt().get(0) != null) {
                        if (cdtTrfTxInf.get(0).getInstrForCdtrAgt().get(0).getCd() != null) {
                            String field23E = cdtTrfTxInf.get(0).getInstrForCdtrAgt().get(0).getCd().value();
                            field23EList.add(field23E);
                        }
                        if (cdtTrfTxInf.get(0).getInstrForCdtrAgt().get(0).getInstrInf() != null) {
                            String field23E = cdtTrfTxInf.get(0).getInstrForCdtrAgt().get(0).getInstrInf();
                            field23EList.add(field23E);
                        }
                    }
                }
            }
            if (field23EList.size() > 0) {
                field23EList.forEach(field23E -> logger.info(":23E:"+field23E));
                map.put("field23EList", field23EList);
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT103 Field23E error");
        }
    }
}
