package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.DateUtil;
import com.prowidesoftware.brilliance.util.NumberUtil;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation11;
import com.prowidesoftware.swift.model.mx.dic.FIToFICustomerCreditTransferV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.datatype.XMLGregorianCalendar;
import java.math.BigDecimal;
import java.util.List;

public class Field32AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field32AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field32AGenerate start.");
        try {
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            String month = "";
            String day = "";
            String year = "";
            if (obj.getFIToFICstmrCdtTrf().getCdtTrfTxInf() != null
                    && obj.getFIToFICstmrCdtTrf().getCdtTrfTxInf().size() >0
                    && obj.getFIToFICstmrCdtTrf().getCdtTrfTxInf().get(0) != null
                    && obj.getFIToFICstmrCdtTrf().getCdtTrfTxInf().get(0).getIntrBkSttlmDt() != null){
                month = String.valueOf(obj.getFIToFICstmrCdtTrf().getCdtTrfTxInf().get(0).getIntrBkSttlmDt().getMonth());
                day = String.valueOf(obj.getFIToFICstmrCdtTrf().getCdtTrfTxInf().get(0).getIntrBkSttlmDt().getDay());
                year = String.valueOf(obj.getFIToFICstmrCdtTrf().getCdtTrfTxInf().get(0).getIntrBkSttlmDt().getYear()) + StringUtil.fillWithChar(month, "0", "left", 2) + StringUtil.fillWithChar(day, "0", "left", 2);
            }
            FIToFICustomerCreditTransferV02 fiToFICstmrCdtTrf = obj.getFIToFICstmrCdtTrf();
            List<CreditTransferTransactionInformation11> cdtTrfTxInf = fiToFICstmrCdtTrf.getCdtTrfTxInf();
            if (cdtTrfTxInf != null && cdtTrfTxInf.size() > 0){
                if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getIntrBkSttlmAmt() != null
                        && cdtTrfTxInf.get(0).getIntrBkSttlmAmt().getCcy() != null
                        && cdtTrfTxInf.get(0).getIntrBkSttlmAmt().getValue() != null){
                    String yymMdd = "";
                    if (cdtTrfTxInf.get(0).getIntrBkSttlmDt() != null){
                        XMLGregorianCalendar dt = cdtTrfTxInf.get(0).getIntrBkSttlmDt();
                        yymMdd = DateUtil.format(dt, "YYMMdd");

                    }
                    String ccy = cdtTrfTxInf.get(0).getIntrBkSttlmAmt().getCcy();
                    BigDecimal amt = cdtTrfTxInf.get(0).getIntrBkSttlmAmt().getValue();
                    String amtStr = NumberUtil.formatAmt(amt, ccy);
                    map.put("field32A",yymMdd+ccy+amtStr);
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT103 Field32A error");
        }

    }
}

