package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation11;
import com.prowidesoftware.swift.model.mx.dic.FIToFICustomerCreditTransferV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 1，如果<AnyBIC>节点有值，则设置50 option为“A”，且BIC为此节点的值
 * 2，若节点<IBAN>有值，则在此节点值前面加上“/”后，作为50的账号；
 * 3，若节点<Othr><Id>有值，则在此节点值前面加上“//CH”后，作为50的账号；
 * <CdtTrfTxInf>[1]<Dbtr><Id><OrgId><AnyBIC>
 * <CdtTrfTxInf>[1]<DbtrAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<DbtrAcct><Id><Othr><Id>
 * 1，待补充？？？
 * 2，账号取值规则与50A相同
 * 1，若<Dbtr><Nm>有录入，则设置50 option为“K”，且名称取自此节点，而地址需要合并截取？？？
 * 2，账号取值规则与50A相同
 * <CdtTrfTxInf>[1]<Dbtr><Nm>
 * <CdtTrfTxInf>[1]<Dbtr><PstlAdr><AdrLine>
 */
public class Field50AGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(Field50AGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        //TODO 为F条件待补充
        logger.info("Field50AGenerate start.");
        try {
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            FIToFICustomerCreditTransferV02 fiToFICstmrCdtTrf = obj.getFIToFICstmrCdtTrf();
            List<CreditTransferTransactionInformation11> cdtTrfTxInf = fiToFICstmrCdtTrf.getCdtTrfTxInf();
            if (cdtTrfTxInf != null && cdtTrfTxInf.size() > 0){
                if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getDbtr() != null
                        && cdtTrfTxInf.get(0).getDbtr().getId() != null
                        && cdtTrfTxInf.get(0).getDbtr().getId().getOrgId() != null
                        && cdtTrfTxInf.get(0).getDbtr().getId().getOrgId().getBICOrBEI() != null){
                    map.put("field50Key","A");
                    map.put("field50A",cdtTrfTxInf.get(0).getDbtr().getId().getOrgId().getBICOrBEI());
                }else if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getDbtr() != null
                        && cdtTrfTxInf.get(0).getDbtr().getNm() != null){
                    map.put("field50Key","K");
                    if (cdtTrfTxInf.get(0).getDbtr() != null
                            && cdtTrfTxInf.get(0).getDbtr().getPstlAdr() != null
                            && cdtTrfTxInf.get(0).getDbtr().getPstlAdr().getAdrLine() != null){
                        List<String> adrList = cdtTrfTxInf.get(0).getDbtr().getPstlAdr().getAdrLine();
                        String adStr = "";
                        for (int i = 0; i<adrList.size(); i++){
                            adStr = adStr + adrList.get(i);
                        }
                        map.put("field50Adr",adStr);
                    }
                }
                if (cdtTrfTxInf.get(0).getDbtrAcct() != null
                        && cdtTrfTxInf.get(0).getDbtrAcct().getId() != null){
                    if (cdtTrfTxInf.get(0).getDbtrAcct().getId().getIBAN() != null){
                        map.put("field50Account","/" + cdtTrfTxInf.get(0).getDbtrAcct().getId().getIBAN());
                    }
                    if (cdtTrfTxInf.get(0).getDbtrAcct().getId().getOthr().getId() != null){
                        map.put("field50Account","//CH" + cdtTrfTxInf.get(0).getDbtrAcct().getId().getOthr().getId());

                    }
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT103 Field50A error");
        }

    }
}
