package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation11;
import com.prowidesoftware.swift.model.mx.dic.FIToFICustomerCreditTransferV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 1，若节点<BICFI>有值，则设置52 option为“A”，同时BIC取此栏位；
 * 2，若节点<IBAN>有值，则作为52的账号；
 * 3，若节点<Othr><SchmeNm><Cd>有值，则取“//CH”+<Othr><Id>，则作为52的账号；否则直接取<Othr><Id>，则作为52的账号。
 * <CdtTrfTxInf>[1]<DbtrAgt><FinInstnId><BICFI>
 * <CdtTrfTxInf>[1]<DbtrAgtAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<DbtrAgtAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<DbtrAgtAcct><Id><Othr><SchmeNm><Cd>
 *  * 1，若节点<Nm>有值，则设置52 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  * 2，账号取值规则和节点与52A相同
 *  * <CdtTrfTxInf>[1]<DbtrAgt><FinInstnId><Nm>
 *  * <CdtTrfTxInf>[1]<DbtrAgt><FinInstnId><PstlAdr><AdrLine>[1]，<AdrLine>[2]，<AdrLine>[3]
 */
public class Field52AGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(Field52AGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field52AGenerate start.");
        try {
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            FIToFICustomerCreditTransferV02 fiToFICstmrCdtTrf = obj.getFIToFICstmrCdtTrf();
            List<CreditTransferTransactionInformation11> cdtTrfTxInf = fiToFICstmrCdtTrf.getCdtTrfTxInf();
            if (cdtTrfTxInf != null && cdtTrfTxInf.size() > 0){
                if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getDbtrAgt() != null
                        && cdtTrfTxInf.get(0).getDbtrAgt().getFinInstnId() != null
                        && cdtTrfTxInf.get(0).getDbtrAgt().getFinInstnId().getBIC() != null){
                    map.put("field52Key","A");
                    map.put("field52A",cdtTrfTxInf.get(0).getDbtrAgt().getFinInstnId().getBIC());
                }else if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getDbtrAgt() != null
                        && cdtTrfTxInf.get(0).getDbtrAgt().getFinInstnId() != null
                        && cdtTrfTxInf.get(0).getDbtrAgt().getFinInstnId().getNm() != null){
                    map.put("field52Key","D");
                    List<String> adrList = cdtTrfTxInf.get(0).getDbtrAgt().getFinInstnId().getPstlAdr().getAdrLine();
                    String adStr = "";
                    for (int i = 0; i<adrList.size(); i++){
                        adStr = adStr + adrList.get(i);
                    }
                    map.put("field52DAdr",adStr);
                }
                if (cdtTrfTxInf.get(0).getDbtrAgtAcct() != null
                        && cdtTrfTxInf.get(0).getDbtrAgtAcct().getId() != null){
                    if (cdtTrfTxInf.get(0).getDbtrAgtAcct().getId().getIBAN() != null){
                        map.put("field52Account",cdtTrfTxInf.get(0).getDbtrAgtAcct().getId().getIBAN());
                    }
                    if (cdtTrfTxInf.get(0).getDbtrAgtAcct().getId().getOthr() != null
                            && cdtTrfTxInf.get(0).getDbtrAgtAcct().getId().getOthr().getSchmeNm() != null
                            && cdtTrfTxInf.get(0).getDbtrAgtAcct().getId().getOthr().getSchmeNm().getCd() != null){
                        if (cdtTrfTxInf.get(0).getDbtrAgtAcct().getId().getOthr().getId() != null){
                            map.put("field52Account","//CH" + cdtTrfTxInf.get(0).getDbtrAgtAcct().getId().getOthr().getId());
                        }
                    }else {
                        if (cdtTrfTxInf.get(0).getDbtrAgtAcct().getId().getOthr().getId() != null){
                            map.put("field52Account",cdtTrfTxInf.get(0).getDbtrAgtAcct().getId().getOthr().getId());
                        }
                    }
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT103 Field52A error");
        }

    }
}