package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.FIToFICustomerCreditTransferV02;
import com.prowidesoftware.swift.model.mx.dic.GroupHeader33;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 1，若节点<BICFI>有值，则设置53 option为“A”，同时BIC取此栏位；
 * 2，若节点<IBAN>有值，则作为53的账号；
 * 3，若节点<Othr><Id>有值，则作为53的账号；
 * <GrpHdr><SttlmInf><InstgRmbrsmntAgt><FinInstnId><BICFI>
 * <GrpHdr><SttlmInf><InstgRmbrsmntAgtAcct><Id><IBAN>
 * <GrpHdr><SttlmInf><InstgRmbrsmntAgtAcct><Id><Othr><Id>
 * <GrpHdr><SttlmInf><InstgRmbrsmntAgtAcct><Id><Othr><SchmeNm><Cd>
 *      * //如果<SttlmAcct><Id><IBAN>或<SttlmAcct><Id><Othr><Id>有值，则设置53 option为“B”：
 *  *         //1）若节点<IBAN>有值，则作为53B的账号；
 *  *         //2）若节点<Othr><Id>有值，则作为53B的账号；
 *  *         //<GrpHdr><SttlmInf><SttlmAcct><Id><IBAN>
 *  *         //<GrpHdr><SttlmInf><SttlmAcct><Id><Othr><Id>
 *  *         //<GrpHdr><SttlmInf><SttlmAcct><Id><Othr><SchmeNm><Cd>
 *      //1，若节点<Nm>有值，则设置53 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  *         //2，账号取值规则和节点与53A相同
 *  *         //<GrpHdr><SttlmInf><InstgRmbrsmntAgt><FinInstnId><Nm>
 *  *         //<GrpHdr><SttlmInf><InstgRmbrsmntAgt><FinInstnId><PstlAdr><AdrLine>[1]，<AdrLine>[2]，<AdrLine>[3]
 */
public class Field53AGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(Field53AGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field53AGenerate start.");
        try {
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            FIToFICustomerCreditTransferV02 fiToFICstmrCdtTrf = obj.getFIToFICstmrCdtTrf();
            GroupHeader33 grpHdr = fiToFICstmrCdtTrf.getGrpHdr();
            if (grpHdr.getSttlmInf() != null){
                if (grpHdr.getSttlmInf().getInstdRmbrsmntAgt() != null
                        && grpHdr.getSttlmInf().getInstdRmbrsmntAgt().getFinInstnId() != null
                        && grpHdr.getSttlmInf().getInstdRmbrsmntAgt().getFinInstnId().getBIC() != null){
                    map.put("field53Key","A");
                    map.put("field53",grpHdr.getSttlmInf().getInstdRmbrsmntAgt().getFinInstnId().getBIC());
                    if (grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct() != null
                            && grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId() != null){
                        if (grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId().getIBAN() != null){
                            map.put("field53Account",grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId().getIBAN());
                        }else if (grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId().getOthr() != null
                                && grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId().getOthr().getId() != null){
                            map.put("field53Account",grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId().getOthr().getId());
                        }
                    }
                }else if ((grpHdr.getSttlmInf().getSttlmAcct() != null
                        && grpHdr.getSttlmInf().getSttlmAcct().getId() != null
                        && grpHdr.getSttlmInf().getSttlmAcct().getId().getIBAN() != null)
                        || (grpHdr.getSttlmInf().getSttlmAcct() != null
                        && grpHdr.getSttlmInf().getSttlmAcct().getId() != null
                        && grpHdr.getSttlmInf().getSttlmAcct().getId().getOthr() != null
                        && grpHdr.getSttlmInf().getSttlmAcct().getId().getOthr().getId() != null)){
                    map.put("field53Key","B");
                    if (grpHdr.getSttlmInf().getSttlmAcct() != null
                            && grpHdr.getSttlmInf().getSttlmAcct().getId() != null
                            && grpHdr.getSttlmInf().getSttlmAcct().getId().getIBAN() != null){
                        map.put("field53Account",grpHdr.getSttlmInf().getSttlmAcct().getId().getIBAN());
                    }else if (grpHdr.getSttlmInf().getSttlmAcct() != null
                            && grpHdr.getSttlmInf().getSttlmAcct().getId() != null
                            && grpHdr.getSttlmInf().getSttlmAcct().getId().getOthr() != null
                            && grpHdr.getSttlmInf().getSttlmAcct().getId().getOthr().getId() != null){
                        map.put("field53Account",grpHdr.getSttlmInf().getSttlmAcct().getId().getOthr().getId());
                    }
                }else if ((grpHdr.getSttlmInf().getInstdRmbrsmntAgt() != null
                        && grpHdr.getSttlmInf().getInstdRmbrsmntAgt().getFinInstnId() != null
                        && grpHdr.getSttlmInf().getInstdRmbrsmntAgt().getFinInstnId().getNm() != null)){
                    map.put("field53Key","D");
                    List<String> adrList = grpHdr.getSttlmInf().getInstdRmbrsmntAgt().getFinInstnId().getPstlAdr().getAdrLine();
                    String adStr = "";
                    for (int i = 0; i<adrList.size(); i++){
                        adStr = adStr + adrList.get(i);
                    }
                    map.put("field53DAdr",adStr);
                    if (grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct() != null
                            && grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId() != null){
                        if (grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId().getIBAN() != null){
                            map.put("field53Account",grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId().getIBAN());
                        }else if (grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId().getOthr() != null
                                && grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId().getOthr().getId() != null){
                            map.put("field53Account",grpHdr.getSttlmInf().getInstdRmbrsmntAgtAcct().getId().getOthr().getId());
                        }
                    }
                }

            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT103 Field53A error");
        }

    }
}
