package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.FIToFICustomerCreditTransferV02;
import com.prowidesoftware.swift.model.mx.dic.GroupHeader33;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * //1，若节点<BICFI>有值，则设置55 option为“A”，同时BIC取此栏位
 *         //2，若节点<IBAN>有值，则作为55的账号；
 *         //3，若节点<Othr><Id>有值，则作为55的账号；
 *         //<GrpHdr><SttlmInf><ThrdRmbrsmntAgt><FinInstnId><BICFI>
 *         //<GrpHdr><SttlmInf><ThrdRmbrsmntAgtAcct><Id><IBAN>
 *         //<GrpHdr><SttlmInf><ThrdRmbrsmntAgtAcct><Id><Othr><Id>
*          * //1，若节点<Nm>有值，则设置55 option为“D”，且名称取自此节点，而地址需要合并截取？？？
*  *       //2，账号取值规则和节点与55A相同
*  *      //<GrpHdr><SttlmInf><ThrdRmbrsmntAgt><FinInstnId><Nm>
*  *      //<GrpHdr><SttlmInf><ThrdRmbrsmntAgt><FinInstnId><PstlAdr><AdrLine>[1]，<AdrLine>[2]，<AdrLine>[3]
 */
public class Field55AGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(com.prowidesoftware.brilliance.mx2mt.mt103.impl.Field55AGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field55AGenerate start.");
        try {
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            FIToFICustomerCreditTransferV02 fiToFICstmrCdtTrf = obj.getFIToFICstmrCdtTrf();
            GroupHeader33 grpHdr = fiToFICstmrCdtTrf.getGrpHdr();
            if (grpHdr.getSttlmInf() != null){
                if (grpHdr.getSttlmInf().getThrdRmbrsmntAgt() != null
                        && grpHdr.getSttlmInf().getThrdRmbrsmntAgt().getFinInstnId() != null
                        && grpHdr.getSttlmInf().getThrdRmbrsmntAgt().getFinInstnId().getBIC() != null){
                    map.put("field55Key","A");
                    map.put("field55",grpHdr.getSttlmInf().getThrdRmbrsmntAgt().getFinInstnId().getBIC());
                }else if ((grpHdr.getSttlmInf().getThrdRmbrsmntAgt() != null
                        && grpHdr.getSttlmInf().getThrdRmbrsmntAgt().getFinInstnId() != null
                        && grpHdr.getSttlmInf().getThrdRmbrsmntAgt().getFinInstnId().getNm() != null)){
                    map.put("field55Key","D");
                    List<String> adrList = grpHdr.getSttlmInf().getInstdRmbrsmntAgt().getFinInstnId().getPstlAdr().getAdrLine();
                    String adStr = "";
                    for (int i = 0; i<adrList.size(); i++){
                        adStr = adStr + adrList.get(i);
                    }
                    map.put("field55DAdr",adStr);
                }
                if (grpHdr.getSttlmInf().getThrdRmbrsmntAgtAcct() != null
                        && grpHdr.getSttlmInf().getThrdRmbrsmntAgtAcct().getId() != null){
                    if (grpHdr.getSttlmInf().getThrdRmbrsmntAgtAcct().getId().getIBAN() != null){
                        map.put("field55Account",grpHdr.getSttlmInf().getThrdRmbrsmntAgtAcct().getId().getIBAN());
                    }else if (grpHdr.getSttlmInf().getThrdRmbrsmntAgtAcct().getId().getOthr() != null
                            && grpHdr.getSttlmInf().getThrdRmbrsmntAgtAcct().getId().getOthr().getId() != null){
                        map.put("field55Account",grpHdr.getSttlmInf().getThrdRmbrsmntAgtAcct().getId().getOthr().getId());
                    }
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT103 Field55A error");
        }

    }
}
