package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation11;
import com.prowidesoftware.swift.model.mx.dic.FIToFICustomerCreditTransferV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * //1，若节点<BICFI>有值，则设置57 option为“A”，同时BIC取此栏位；
 *         //2，若节点<IBAN>有值，则作为57A的账号；
 *         //3，若节点<Othr><Id>有值，则作为57A的账号；
 *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><BICFI>
 *         //<CdtTrfTxInf>[1]<CdtrAgtAcct><Id><IBAN>
 *         //<CdtTrfTxInf>[1]<CdtrAgtAcct><Id><Othr><Id>
 *         //<CdtTrfTxInf>[1]<CdtrAgtAcct><Id><Othr><SchmeNm><Cd>
 *              * //如果<CdtrAgt><FinInstnId><BICFI>为空，且<CdtrAgt><FinInstnId><Nm>为空，
 *  *         // 但节点<IBAN>有值，或者节点<Othr><Id>有值，则设置57 option为“C”
 *  *         //若节点<Id><Othr><Id>不为空，则取“//CH”+此节点作为账号，否则取节点<MmbId>作为账号：
 *  *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><ClrSysMmbId><ClrSysId><Cd>
 *  *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><ClrSysMmbId><ClrSysId><Prtry>
 *  *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><ClrSysMmbId><MmbId>
 *  *         //<CdtTrfTxInf>[1]<CdtrAgtAcct><Id><Othr><Id>
 *       *         //1，若节点<Nm>有值，则设置57 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  *         //2，账号取值规则和节点与57A相同
 *  *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><Nm>
 *  *         //<CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><PstlAdr><AdrLine>[1],<AdrLine>[2],<AdrLine>[3]
 */
public class Field57AGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(com.prowidesoftware.brilliance.mx2mt.mt103.impl.Field57AGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field57AGenerate start.");
        try {
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            FIToFICustomerCreditTransferV02 fiToFICstmrCdtTrf = obj.getFIToFICstmrCdtTrf();
            List<CreditTransferTransactionInformation11> cdtTrfTxInf = fiToFICstmrCdtTrf.getCdtTrfTxInf();
            if (cdtTrfTxInf != null && cdtTrfTxInf.size() > 0){
                if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getCdtrAgt() != null
                        && cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId() != null
                        && cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId().getBIC() != null){
                    map.put("field57Key","A");
                    map.put("field57A",cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId().getBIC());
                    if (cdtTrfTxInf.get(0).getCdtrAgtAcct() != null
                            && cdtTrfTxInf.get(0).getCdtrAgtAcct().getId() != null){
                        if (cdtTrfTxInf.get(0).getCdtrAgtAcct().getId().getIBAN() != null){
                            map.put("field57Account",cdtTrfTxInf.get(0).getCdtrAgtAcct().getId().getIBAN());
                        }else if (cdtTrfTxInf.get(0).getCdtrAgtAcct().getId().getOthr() != null
                                && cdtTrfTxInf.get(0).getCdtrAgtAcct().getId().getOthr().getId() != null){
                            map.put("field57Account",cdtTrfTxInf.get(0).getCdtrAgtAcct().getId().getOthr().getId());
                        }
                    }
                }else if ((cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getCdtrAgt() != null
                        && cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId() != null
                        && cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId().getBIC() == null)
                        && (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getCdtrAgt() != null
                        && cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId() != null
                        && cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId().getNm() == null)){
                    if ((cdtTrfTxInf.get(0) != null
                            && cdtTrfTxInf.get(0).getCdtrAcct() != null
                            && cdtTrfTxInf.get(0).getCdtrAcct().getId() != null
                            && cdtTrfTxInf.get(0).getCdtrAcct().getId().getIBAN() != null)
                            || (cdtTrfTxInf.get(0) != null
                            && cdtTrfTxInf.get(0).getCdtrAcct() != null
                            && cdtTrfTxInf.get(0).getCdtrAcct().getId() != null
                            && cdtTrfTxInf.get(0).getCdtrAcct().getId().getOthr() != null
                            && cdtTrfTxInf.get(0).getCdtrAcct().getId().getOthr().getId() != null)){
                        map.put("field57Key","C");
                        if (cdtTrfTxInf.get(0) != null
                                && cdtTrfTxInf.get(0).getCdtrAcct() != null
                                && cdtTrfTxInf.get(0).getCdtrAcct().getId() != null
                                && cdtTrfTxInf.get(0).getCdtrAcct().getId().getOthr() != null
                                && cdtTrfTxInf.get(0).getCdtrAcct().getId().getOthr().getId() != null){
                            map.put("field57Account",cdtTrfTxInf.get(0).getCdtrAcct().getId().getOthr().getId());
                        }else if (cdtTrfTxInf.get(0) != null
                                && cdtTrfTxInf.get(0).getCdtrAgt() != null
                                && cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId() != null
                                && cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId().getClrSysMmbId() != null
                                && cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId().getClrSysMmbId().getMmbId() != null){
                            map.put("field57Account",cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId().getClrSysMmbId().getMmbId());
                        }
                    }
                }else if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getCdtrAgt() != null
                        && cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId() != null
                        && cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId().getNm() != null){
                    map.put("field57Key","D");
                    List<String> adrList = cdtTrfTxInf.get(0).getCdtrAgt().getFinInstnId().getPstlAdr().getAdrLine();
                    String adStr = "";
                    for (int i = 0; i<adrList.size(); i++){
                        adStr = adStr + adrList.get(i);
                    }
                    map.put("field57DAdr",adStr);
                    if (cdtTrfTxInf.get(0).getCdtrAgtAcct() != null
                            && cdtTrfTxInf.get(0).getCdtrAgtAcct().getId() != null){
                        if (cdtTrfTxInf.get(0).getCdtrAgtAcct().getId().getIBAN() != null){
                            map.put("field57Account",cdtTrfTxInf.get(0).getCdtrAgtAcct().getId().getIBAN());
                        }
                        if (cdtTrfTxInf.get(0).getCdtrAgtAcct().getId().getOthr() != null
                                && cdtTrfTxInf.get(0).getCdtrAgtAcct().getId().getOthr().getId() != null){
                            map.put("field57Account",cdtTrfTxInf.get(0).getCdtrAgtAcct().getId().getOthr().getId());
                        }
                    }
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT103 Field57A error");
        }

    }
}
