package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation11;
import com.prowidesoftware.swift.model.mx.dic.FIToFICustomerCreditTransferV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 1）如果节点值为 'CRED'，则71A取值为“BEN”
 * 2）如果节点值为 'DEBT'，则71A取值为“OUR”
 * 3）如果节点值为'SHAR'，则71A取值为“SHA”
 * <CdtTrfTxInf>[1]<ChrgBr>
 *      * //1，若71A为“BEN”或“SHA”，则此节点取值为71F
 *  *         //CdtTrfTxInf>[1]<ChrgsInf><Amt Ccy="XXX">
 *       * 1，若71A为“OUR”，则此节点取值为71G；
 *  * <CdtTrfTxInf>[1]<ChrgsInf><Amt Ccy="XXX">
 */
public class Field71AGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(com.prowidesoftware.brilliance.mx2mt.mt103.impl.Field71AGenerate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field71AGenerate start.");
        try {
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            FIToFICustomerCreditTransferV02 fiToFICstmrCdtTrf = obj.getFIToFICstmrCdtTrf();
            List<CreditTransferTransactionInformation11> cdtTrfTxInf = fiToFICstmrCdtTrf.getCdtTrfTxInf();
            if (cdtTrfTxInf != null && cdtTrfTxInf.size() > 0){
                if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getChrgBr() != null
                        && cdtTrfTxInf.get(0).getChrgBr().value() != null) {
                    String field71A_mx = cdtTrfTxInf.get(0).getChrgBr().value();
                    String field71A="";
                    if("CRED".equals(field71A_mx))
                    {
                        field71A="BEN";
                    }else if("DEBT".equals(field71A_mx)) {
                        field71A="OUR";
                    }else if("SHAR".equals(field71A_mx)) {
                        field71A="SHA";
                    }
                    map.put("field71A",field71A);
                }else if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getChrgsInf().size() >0
                        && cdtTrfTxInf.get(0).getChrgsInf().get(0) != null
                        && cdtTrfTxInf.get(0).getChrgsInf().get(0).getAmt() != null
                        && cdtTrfTxInf.get(0).getChrgsInf().get(0).getAmt().getCcy() != null) {
                    if ("BEN".equals(map.get("field71A")) || "SHA".equals(map.get("field71A"))){
                        map.put("field71F",cdtTrfTxInf.get(0).getChrgsInf().get(0).getAmt().getCcy());
                    }
                }else if (cdtTrfTxInf != null && cdtTrfTxInf.size() > 0){
                    if (cdtTrfTxInf.get(0) != null
                            && cdtTrfTxInf.get(0).getChrgsInf().size() > 0
                            && cdtTrfTxInf.get(0).getChrgsInf().get(0) != null
                            && cdtTrfTxInf.get(0).getChrgsInf().get(0).getAmt() != null
                            && cdtTrfTxInf.get(0).getChrgsInf().get(0).getAmt().getCcy() != null) {
                        if ("OUR".equals(map.get("field71A"))){
                            map.put("field71G",cdtTrfTxInf.get(0).getChrgsInf().get(0).getAmt().getCcy());
                        }
                    }
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT103 Field71A error");
        }

    }
}
