package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation11;
import com.prowidesoftware.swift.model.mx.dic.FIToFICustomerCreditTransferV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * //1，如果<BICFI>有值，则取值为“/INS/”+此节点的值
 *         //2，如果<Nm>有值，则取值为“/INS/”+此节点的值，超过35字节则将剩下内容用“//内容“的方式进行截取
 *         //3，若<InstrForCdtrAgt><InstrInf>有值，则转为“/ACC/此节点的值”，超过35字节则将剩下内容用“//内容“的方式进行截取
 *         //4，若<InstrForNxtAgt><InstrInf>有值，则转为“/REC/此节点的值”，超过35字节则将剩下内容用“//内容“的方式进行截取
 *         //<CdtTrfTxInf>[1]<PrvsInstgAgt1><FinInstnId><BICFI>
 *         //<CdtTrfTxInf>[1]<PrvsInstgAgt1><FinInstnId><Nm>
 *         //<CdtTrfTxInf>[1]<InstrForCdtrAgt><InstrInf>
 *         //<CdtTrfTxInf>[1]<InstrForNxtAgt><InstrInf>
 */
public class Field72Generate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(com.prowidesoftware.brilliance.mx2mt.mt103.impl.Field72Generate.class);
    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field72Generate start.");
        try {
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            FIToFICustomerCreditTransferV02 fiToFICstmrCdtTrf = obj.getFIToFICstmrCdtTrf();
            List<CreditTransferTransactionInformation11> cdtTrfTxInf = fiToFICstmrCdtTrf.getCdtTrfTxInf();
            if (cdtTrfTxInf != null && cdtTrfTxInf.size() > 0){
                if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getPrvsInstgAgt() != null
                        && cdtTrfTxInf.get(0).getPrvsInstgAgt().getFinInstnId() != null){
                    if (cdtTrfTxInf.get(0).getPrvsInstgAgt().getFinInstnId().getBIC() != null) {
                        map.put("field72Adr1","/INS/" + cdtTrfTxInf.get(0).getPrvsInstgAgt().getFinInstnId().getBIC());
                    }
                    if (cdtTrfTxInf.get(0).getPrvsInstgAgt().getFinInstnId().getNm() != null){
                        String str = cdtTrfTxInf.get(0).getPrvsInstgAgt().getFinInstnId().getNm();
                        String strs = cutForStr(str,35);
                        map.put("field72Adr2","/INS/" + strs);
                    }
                }
                if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getInstrForCdtrAgt().size() > 0
                        && cdtTrfTxInf.get(0).getInstrForCdtrAgt().get(0) != null
                        && cdtTrfTxInf.get(0).getInstrForCdtrAgt().get(0).getInstrInf() != null){
                    String str = cdtTrfTxInf.get(0).getInstrForCdtrAgt().get(0).getInstrInf();
                    String strs = cutForStr(str,35);
                    map.put("field72Adr3","/ACC/" + strs);
                }
                if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getInstrForNxtAgt().size() > 0
                        && cdtTrfTxInf.get(0).getInstrForNxtAgt().get(0) != null
                        && cdtTrfTxInf.get(0).getInstrForNxtAgt().get(0).getInstrInf() != null){
                    String str = cdtTrfTxInf.get(0).getInstrForNxtAgt().get(0).getInstrInf();
                    String strs = cutForStr(str,35);
                    map.put("field72Adr4","/REC/" + strs);
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT103 Field72 error");
        }

    }
    //循环截取多少位字节并以//隔开
    public String cutForStr(String str,int n){
        String line = "";
        byte[] b1 = str.getBytes();
        for (int i = 0; i <= Math.ceil(b1.length/n); i++){
            int b = str.getBytes().length;
            if (b > 15){
                String s = cutBeginStr(str,n);
                line = line + s + "//";
                str = str.substring(s.length());
            }else{
                line = line + str;
            }
        }
        return line;
    }

    //截取多少位字节
    public static String cutBeginStr(String strs,int length){
        int sum = 0;
        String finalStr = "";
        if (null == strs || strs.getBytes().length <= length){
            finalStr = (strs == null?"":strs);
        }else{
            for (int i = 0;i < strs.length();i++){
                String str = strs.substring(i,i+1);
                //累加单个字符字节数
                sum += str.getBytes().length;
                if (sum > length){
                    finalStr = strs.substring(0,i);
                    break;
                }
            }
        }
        return finalStr;
    }
}

