package com.prowidesoftware.brilliance.mx2mt.mt103.impl;


import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.GroupHeader33;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ReceiverBicFieldGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(ReceiverBicFieldGenerate.class);

    /**
     * <GrpHdr><InstgAgt><FinInstnId><BICFI>
     */
    @Override
    public void fieldsGenerate() {
        logger.info("ReceiverBicFieldGenerate start.");
        try{
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            GroupHeader33 grpHdr = obj.getFIToFICstmrCdtTrf().getGrpHdr();
            if (grpHdr != null) {
                if (grpHdr.getInstgAgt() != null
                    && grpHdr.getInstgAgt().getFinInstnId() != null
                    && grpHdr.getInstgAgt().getFinInstnId().getBIC() != null){
                    String receiverBic = grpHdr.getInstgAgt().getFinInstnId().getBIC();
                    logger.info("Receiver Bic : " + receiverBic);
                    if (!StringUtil.isEmpty(receiverBic)) {
                        String mtReceiverBic = "";
                        if (receiverBic.length() == 8) {
                            mtReceiverBic = receiverBic + Mx2MtConstants.BICSUFFIX;
                        } else if (receiverBic.length() == 11) {
                            mtReceiverBic = receiverBic.substring(0, 8) + Mx2MtConstants.BICMIDDLE + receiverBic.substring(8);
                        } else {
                            throw new SwiftException("ERROR", "ReceiverBic length is invalid.");
                        }
                        map.put("ReceiverBic", mtReceiverBic);
                    }
                }
            }

        }catch(Exception e){
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT103 ReceiverBic error.");
        }
    }
}
