package com.prowidesoftware.brilliance.mx2mt.mt103.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;

import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00800102;
import com.prowidesoftware.swift.model.mx.dic.GroupHeader33;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SenderBicFieldGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(ReceiverBicFieldGenerate.class);

    /**
     * <GrpHdr><InstgAgt><FinInstnId><BICFI>
     */
    @Override
    public void fieldsGenerate() {
        logger.info("SenderBicFieldGenerate start.");
        try{
            MxPacs00800102 obj =  (MxPacs00800102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFIToFICstmrCdtTrf() == null) {
                return;
            }
            GroupHeader33 grpHdr = obj.getFIToFICstmrCdtTrf().getGrpHdr();
            if (grpHdr != null) {
                if (grpHdr.getInstgAgt() != null
                        && grpHdr.getInstgAgt().getFinInstnId() != null
                        && grpHdr.getInstgAgt().getFinInstnId().getBIC() != null){
                    String senderBic = grpHdr.getInstgAgt().getFinInstnId().getBIC();
                    logger.info("Sender Bic : " + senderBic);
                    if (!StringUtil.isEmpty(senderBic)) {
                        String mtsenderBic = "";
                        if (senderBic.length() == 8) {
                            mtsenderBic = senderBic + Mx2MtConstants.BICSUFFIX;
                        } else if (senderBic.length() == 11) {
                            mtsenderBic = senderBic.substring(0, 8) + Mx2MtConstants.BICMIDDLE + senderBic.substring(8);
                        } else {
                            throw new SwiftException("ERROR", "senderBic length is invalid.");
                        }
                        map.put("SenderBic", mtsenderBic);
                    }
                }
            }
        }catch(Exception e){
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT103 SenderBic error.");
        }
    }
}
