package com.prowidesoftware.brilliance.mx2mt.mt202cov202;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtCreator;
import com.prowidesoftware.brilliance.mx2mt.Mx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl.*;
import com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl.cov.*;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.brilliance.util.TemplateUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00900102;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation13;
import com.prowidesoftware.swift.model.mx.dic.FinancialInstitutionCreditTransferV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class Mx2Mt202Creator extends AbstractMx2MtCreator {
    private static final Logger logger = LoggerFactory.getLogger(Mx2Mt202Creator.class);

    @Override
    public List<Mx2MtFieldsGenerate> getFieldsGenerateList() {
        List<Mx2MtFieldsGenerate> fieldsGenerateList = new ArrayList<>();
        fieldsGenerateList.add(new Field20Generate());
        fieldsGenerateList.add(new Field21Generate());
        fieldsGenerateList.add(new Field13CGenerate());
        fieldsGenerateList.add(new Field32AGenerate());
        fieldsGenerateList.add(new Field52AGenerate());
        fieldsGenerateList.add(new Field53AGenerate());
        fieldsGenerateList.add(new Field54AGenerate());
        fieldsGenerateList.add(new Field56AGenerate());
        fieldsGenerateList.add(new Field57AGenerate());
        fieldsGenerateList.add(new Field58AGenerate());
        fieldsGenerateList.add(new Field72Generate());
        fieldsGenerateList.add(new Field50AGenerate());
        fieldsGenerateList.add(new FieldB52AGenerate());
        fieldsGenerateList.add(new FieldB56Generate());
        fieldsGenerateList.add(new FieldB57Generate());
        fieldsGenerateList.add(new FieldB59Generate());
        fieldsGenerateList.add(new FieldB70Generate());
        fieldsGenerateList.add(new FieldB72Generate());
        fieldsGenerateList.add(new FieldB33BGenerate());
        return fieldsGenerateList;
    }

    @Override
    public String generateMtMsg() {
        String outPutFilePath = (String)map.get(Mx2MtConstants.OUT_PUT_FILE_PATH);
        String msg = TemplateUtil.makeFileByTemplate("swift_mt_202cov.vm", map, outPutFilePath, "utf-8");
        return msg;
    }

    /**
     * 判断使用202模板或者202cov模板
     * 依据202cov必有值项50有值的话则使用202cov的模板
     * @return true：202cov，false：202
     */
    public boolean getMtCov(){
        Boolean msg = false;
        try {
            MxPacs00900102 obj =  (MxPacs00900102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFinInstnCdtTrf() == null) {
                msg = false;
            }
            FinancialInstitutionCreditTransferV02 finInstnCdtTrf = obj.getFinInstnCdtTrf();
            List<CreditTransferTransactionInformation13> cdtTrfTxInf = finInstnCdtTrf.getCdtTrfTxInf();
            if (cdtTrfTxInf.size() > 0
                    && cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf() != null
                    && cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf().getDbtr() != null
                    && cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf().getDbtr().getId() != null
                    && cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf().getDbtr().getId().getOrgId() != null
                    && StringUtil.isNotEmpty(cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf().getDbtr().getId().getOrgId().getBICOrBEI())){
                msg = true;
            }
            if (cdtTrfTxInf.size() > 0
                    && cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf() != null
                    && cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf().getDbtr() != null
                    && StringUtil.isNotEmpty(cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf().getDbtr().getNm())){
                msg = true;
            }
        } catch(Exception e) {
            throw new SwiftException("ERROR", "Judge 202 or 202Cov error");
        }
        return msg;
    }

    @Override
    protected String getMtType() {
        return Mx2MtConstants.MT_TYPE_202;
    }

    @Override
    public void generateHead3() {
        String head3 = "";
        if (getMtCov()) {
            head3 = Mx2MtConstants.HEAD3_MT202_COVER;
        }
        map.put("head3", head3);
    }
}