package com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.DateUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00900102;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation13;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.datatype.XMLGregorianCalendar;
import java.util.ArrayList;
import java.util.List;

/**
 * 1，如果<DbtDtTm>节点有值，则转为/SNDTIME/HHMM+0800，HHMM是此节点中的时间，+0800是香港时区
 * 2，如果<CdtDtTm>节点有值，则转为/RNCTIME/HHMM+0800，HHMM是此节点中的时间，+0800是香港时区
 * 3，如果<CLSTm>节点有值，则转为/CLSTIME/HHMM+0800，HHMM是此节点中的时间，+0800是香港时区
 * <CdtTrfTxInf>[1]<SttlmTmIndctn><DbtDtTm>
 * <CdtTrfTxInf>[1]<SttlmTmIndctn><CdtDtTm>
 * <CdtTrfTxInf>[1]<SttlmTmReq><CLSTm>
 */
public class Field13CGenerate extends AbstractMx2MtFieldsGenerate {
    private static final Logger logger = LoggerFactory.getLogger(com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl.Field13CGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            MxPacs00900102 obj =  (MxPacs00900102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFinInstnCdtTrf() == null) {
                return;
            }
            List<CreditTransferTransactionInformation13> cdtTrfTxInf = obj.getFinInstnCdtTrf().getCdtTrfTxInf();
            List<String> field13cList = new ArrayList<>();
            if (obj.getFinInstnCdtTrf().getCdtTrfTxInf().get(0).getSttlmTmIndctn().getDbtDtTm() != null){
                XMLGregorianCalendar dbtDtTm = cdtTrfTxInf.get(0).getSttlmTmIndctn().getDbtDtTm();
                String hmm = DateUtil.format(dbtDtTm, "HHmm");
                field13cList.add("/SNDTIME/"+hmm+"+0800");
            }
            if (obj.getFinInstnCdtTrf().getCdtTrfTxInf().get(0).getSttlmTmIndctn().getCdtDtTm() != null){
                XMLGregorianCalendar dbtDtTm = cdtTrfTxInf.get(0).getSttlmTmIndctn().getCdtDtTm();
                String hmm = DateUtil.format(dbtDtTm, "HHmm");
                field13cList.add("/RNCTIME/"+hmm+"+0800");
            }
            if (obj.getFinInstnCdtTrf().getCdtTrfTxInf().get(0).getSttlmTmReq() != null
                    && obj.getFinInstnCdtTrf().getCdtTrfTxInf().get(0).getSttlmTmReq().getCLSTm() != null){
                XMLGregorianCalendar dbtDtTm = cdtTrfTxInf.get(0).getSttlmTmReq().getCLSTm();
                String hmm = DateUtil.format(dbtDtTm, "HHmm");
                field13cList.add("/CLSTIME/"+hmm+"+0800");
            }
            if (field13cList.size() > 0) {
                map.put("field13CList", field13cList);
            }
        } catch(Exception e) {
            throw new SwiftException("ERROR", "Convert MT202 Field13C error");
        }

    }
}

