package com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.DateUtil;
import com.prowidesoftware.brilliance.util.NumberUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00900102;
import com.prowidesoftware.swift.model.mx.dic.ActiveCurrencyAndAmount;
import com.prowidesoftware.swift.model.mx.dic.CreditTransferTransactionInformation13;
import com.prowidesoftware.swift.model.mx.dic.FinancialInstitutionCreditTransferV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.datatype.XMLGregorianCalendar;
import java.math.BigDecimal;
import java.util.List;

/**
 * 直接取<IntrBkSttlmDt>节点作为32A的value date，Amt和Ccy看另一个节点，XXX是货币
 * <CdtTrfTxInf>[1]<IntrBkSttlmDt>
 * <CdtTrfTxInf>[1]<IntrBkSttlmAmt Ccy="XXX"'>
 */
public class Field32AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl.Field32AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            MxPacs00900102 obj =  (MxPacs00900102)map.get(Mx2MtConstants.MX_OBJECT);
            if (obj == null || obj.getFinInstnCdtTrf() == null) {
                return;
            }
            FinancialInstitutionCreditTransferV02 finInstnCdtTrf = obj.getFinInstnCdtTrf();
            List<CreditTransferTransactionInformation13> cdtTrfTxInf = finInstnCdtTrf.getCdtTrfTxInf();
            if (cdtTrfTxInf != null && cdtTrfTxInf.size() > 0){
                if (cdtTrfTxInf.get(0) != null
                        && cdtTrfTxInf.get(0).getIntrBkSttlmAmt() != null
                        && cdtTrfTxInf.get(0).getIntrBkSttlmAmt().getCcy() != null
                        && cdtTrfTxInf.get(0).getIntrBkSttlmAmt().getValue() != null){
                    String yyMMdd = "";
                    if (cdtTrfTxInf.get(0).getIntrBkSttlmDt() != null){
                        XMLGregorianCalendar dt = cdtTrfTxInf.get(0).getIntrBkSttlmDt();
                        yyMMdd = DateUtil.format(dt, "yyMMdd");
                    }
                    ActiveCurrencyAndAmount intrBkSttlmAmt = cdtTrfTxInf.get(0).getIntrBkSttlmAmt();
                    String ccy = "";
                    String amtStr = "";
                    if (intrBkSttlmAmt != null) {
                        ccy = intrBkSttlmAmt.getCcy();
                        BigDecimal amt = intrBkSttlmAmt.getValue();
                        amtStr = NumberUtil.formatAmt(amt, ccy);
                    }
                    map.put("field32A",yyMMdd+ccy+amtStr);
                }
            }
        } catch(Exception e) {
            throw new SwiftException("ERROR", "Convert MT202 Field32A error");
        }

    }
}
