package com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl;


import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00900102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 1，若节点<BICFI>有值，则设置52 option为“A”，同时BIC取此栏位
 * 2，若节点<IBAN>有值，则作为52A的账号；
 * 3，若节点<Othr><SchmeNm><Cd>有值，则取“//CH”+<Othr><Id>，则作为52A的账号；否则直接取<Othr><Id>，则作为52A的账号
 * <CdtTrfTxInf>[1]<Dbtr><FinInstnId><BICFI>
 * <CdtTrfTxInf>[1]<DbtrAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<DbtrAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<DbtrAcct><Id><Othr><SchmeNm><Cd>
 */
public class Field52AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field52AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            CreditTransferTransactionInformation13 cdtTrfTxInf = getCreditTransferTransactionInformation13();
            String field52Option = "";
            String field52A = "";
            if (cdtTrfTxInf != null) {
                BranchAndFinancialInstitutionIdentification4 dbtr = cdtTrfTxInf.getDbtr();
                if (dbtr != null) {
                    FinancialInstitutionIdentification7 finInstnId = dbtr.getFinInstnId();
                    if (finInstnId != null) {
                        if (StringUtil.isNotEmpty(finInstnId.getBIC())) {
                            field52Option = "A";
                            String bicCode = finInstnId.getBIC();
                            String account = getMtFormatAccount(cdtTrfTxInf.getDbtrAcct());
                            if (!StringUtil.isEmpty(account)) {
                                field52A = account + Mx2MtConstants.NEW_LINE + bicCode;
                            } else {
                                field52A = bicCode;
                            }
                        } else if (StringUtil.isNotEmpty(finInstnId.getNm())) {
                            field52Option = "D";
                            String nameAddr = "";
                            PostalAddress6 pstlAdr = finInstnId.getPstlAdr();
                            if (pstlAdr != null) {
                                List<String> adrLineList = pstlAdr.getAdrLine();
                                if (adrLineList.size() > 0) {
                                    for (int i=0; i<adrLineList.size(); i++) {
                                        if (i == 4) {
                                            break;
                                        }
                                        String adrLine = adrLineList.get(i);
                                        if (adrLine.length() > 35) {
                                            adrLine = adrLine.substring(0, 35);
                                        }
                                        if (i == 0) {
                                            nameAddr += adrLine;
                                        } else {
                                            nameAddr += Mx2MtConstants.NEW_LINE+adrLine;
                                        }
                                    }
                                }
                            }
                            String account = getMtFormatAccount(cdtTrfTxInf.getDbtrAcct());
                            if (!StringUtil.isEmpty(account)) {
                                field52A = account + Mx2MtConstants.NEW_LINE + nameAddr;;
                            } else {
                                field52A = nameAddr;
                            }
                        }
                    }
                }
            }
            map.put("field52Option", field52Option);
            map.put("field52A", field52A);
        } catch(Exception e) {
            throw new SwiftException("ERROR", "Convert MT202 Field52A error");
        }

    }

    public CreditTransferTransactionInformation13 getCreditTransferTransactionInformation13() {
        MxPacs00900102 obj =  (MxPacs00900102)map.get(Mx2MtConstants.MX_OBJECT);
        if (obj == null || obj.getFinInstnCdtTrf() == null) {
            return null;
        }
        FinancialInstitutionCreditTransferV02 finInstnCdtTrf = obj.getFinInstnCdtTrf();
        List<CreditTransferTransactionInformation13> cdtTrfTxInf = finInstnCdtTrf.getCdtTrfTxInf();
        if (cdtTrfTxInf.size() > 0) {
            return cdtTrfTxInf.get(0);
        } else {
            return null;
        }
    }
}
