package com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00900102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 1,若节点<BICFI>有值，则设置54 option为“A”，同时BIC取此栏位；
 * 2,若节点<IBAN>有值，则作为54A的账号；
 * 3,若节点<Othr><Id>有值，则作为54A的账号；
 * <GrpHdr><SttlmInf><InstdRmbrsmntAgt><FinInstnId><BICFI>
 * <GrpHdr><SttlmInf><InstdRmbrsmntAgtAcct><Id><IBAN>
 * <GrpHdr><SttlmInf><InstdRmbrsmntAgtAcct><Id><Othr><Id>
 * <GrpHdr><SttlmInf><InstdRmbrsmntAgtAcct><Id><Othr><SchmeNm><Cd>
 * 1,若节点<Nm>有值，则设置54 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  * 2,账号取值规则和节点与54A相同
 *  * <GrpHdr><SttlmInf><InstdRmbrsmntAgt><FinInstnId><Nm>
 *  * <GrpHdr><SttlmInf><InstdRmbrsmntAgt><FinInstnId><PstlAdr><AdrLine>[1]，<AdrLine>[2]，<AdrLine>[3]
 *
 */
public class Field54AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field54AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            SettlementInformation13 sttlmInf = getSttlmInf();
            String field54Option = "";
            String field54A = "";
            if (sttlmInf != null) {
                BranchAndFinancialInstitutionIdentification4 instdRmbrsmntAgt = sttlmInf.getInstdRmbrsmntAgt();
                if (instdRmbrsmntAgt != null) {
                    FinancialInstitutionIdentification7 finInstnId = instdRmbrsmntAgt.getFinInstnId();
                    if (finInstnId != null) {
                        if (StringUtil.isNotEmpty(finInstnId.getBIC())) {
                            field54Option = "A";
                            String bicCode = finInstnId.getBIC();
                            String account = getMtFormatAccount(sttlmInf.getInstdRmbrsmntAgtAcct());
                            if (!StringUtil.isEmpty(account)) {
                                field54A = account + Mx2MtConstants.NEW_LINE + bicCode;
                            } else {
                                field54A = bicCode;
                            }
                        } else if (StringUtil.isNotEmpty(finInstnId.getNm())) {
                            field54Option = "D";
                            String nameAddr = "";
                            PostalAddress6 pstlAdr = finInstnId.getPstlAdr();
                            if (pstlAdr != null) {
                                List<String> adrLineList = pstlAdr.getAdrLine();
                                if (adrLineList.size() > 0) {
                                    for (int i=0; i<adrLineList.size(); i++) {
                                        if (i == 4) {
                                            break;
                                        }
                                        String adrLine = adrLineList.get(i);
                                        if (adrLine.length() > 35) {
                                            adrLine = adrLine.substring(0, 35);
                                        }
                                        if (i == 0) {
                                            nameAddr += adrLine;
                                        } else {
                                            nameAddr += Mx2MtConstants.NEW_LINE+adrLine;
                                        }
                                    }
                                }
                            }
                            String account = getMtFormatAccount(sttlmInf.getInstdRmbrsmntAgtAcct());
                            if (!StringUtil.isEmpty(account)) {
                                field54A = account + Mx2MtConstants.NEW_LINE + nameAddr;;
                            } else {
                                field54A = nameAddr;
                            }
                        }
                    }
                }
            }
            map.put("field54Option", field54Option);
            map.put("field54A", field54A);
        } catch(Exception e) {
            throw new SwiftException("ERROR", "Convert MT202 Field54A error");
        }

    }

    public SettlementInformation13 getSttlmInf() {
        SettlementInformation13 sttlmInf = null;
        MxPacs00900102 obj =  (MxPacs00900102)map.get(Mx2MtConstants.MX_OBJECT);
        if (obj == null || obj.getFinInstnCdtTrf() == null) {
            return null;
        }
        FinancialInstitutionCreditTransferV02 finInstnCdtTrf = obj.getFinInstnCdtTrf();
        if (finInstnCdtTrf.getGrpHdr() != null) {
            sttlmInf = finInstnCdtTrf.getGrpHdr().getSttlmInf();
        }
        return sttlmInf;
    }
}
