package com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00900102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 1,若节点<BICFI>有值，则设置57 option为“A”，同时BIC取此栏位；
 * 2,若节点<IBAN>有值，则作为57A的账号；
 * 3,若节点<Othr><Id>有值，则作为57A的账号；
 * <CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><BICFI>
 * <CdtTrfTxInf>[1]<CdtrAgtAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<CdtrAgtAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<CdtrAgtAcct><Id><Othr><SchmeNm><Cd>
 *      * 1,若节点<Nm>有值，则设置57 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  * 2,账号取值规则和节点与57A相同
 *  * <CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><Nm>
 *  * <CdtTrfTxInf>[1]<CdtrAgt><FinInstnId><PstlAdr><AdrLine>[1],<AdrLine>[2],<AdrLine>[3]
 */
public class Field57AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field57AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            CreditTransferTransactionInformation13 cdtTrfTxInf = getCreditTransferTransactionInformation13();
            String field57Option = "";
            String field57A = "";
            if (cdtTrfTxInf != null) {
                BranchAndFinancialInstitutionIdentification4 dbtr = cdtTrfTxInf.getCdtrAgt();
                if (dbtr != null) {
                    FinancialInstitutionIdentification7 finInstnId = dbtr.getFinInstnId();
                    if (finInstnId != null) {
                        if (StringUtil.isNotEmpty(finInstnId.getBIC())) {
                            field57Option = "A";
                            String bicCode = finInstnId.getBIC();
                            String account = getMtFormatAccount(cdtTrfTxInf.getCdtrAgtAcct());
                            if (!StringUtil.isEmpty(account)) {
                                field57A = account + Mx2MtConstants.NEW_LINE + bicCode;
                            } else {
                                field57A = bicCode;
                            }
                        } else if (StringUtil.isNotEmpty(finInstnId.getNm())) {
                            field57Option = "D";
                            String nameAddr = "";
                            PostalAddress6 pstlAdr = finInstnId.getPstlAdr();
                            if (pstlAdr != null) {
                                List<String> adrLineList = pstlAdr.getAdrLine();
                                if (adrLineList.size() > 0) {
                                    for (int i=0; i<adrLineList.size(); i++) {
                                        if (i == 4) {
                                            break;
                                        }
                                        String adrLine = adrLineList.get(i);
                                        if (adrLine.length() > 35) {
                                            adrLine = adrLine.substring(0, 35);
                                        }
                                        if (i == 0) {
                                            nameAddr += adrLine;
                                        } else {
                                            nameAddr += Mx2MtConstants.NEW_LINE+adrLine;
                                        }
                                    }
                                }
                            }
                            String account = getMtFormatAccount(cdtTrfTxInf.getCdtrAgtAcct());
                            if (!StringUtil.isEmpty(account)) {
                                field57A = account + Mx2MtConstants.NEW_LINE + nameAddr;;
                            } else {
                                field57A = nameAddr;
                            }
                        }
                    }
                }
            }
            map.put("field57Option", field57Option);
            map.put("field57A", field57A);
        } catch(Exception e) {
            throw new SwiftException("ERROR", "Convert MT202 Field57A error");
        }

    }

    public CreditTransferTransactionInformation13 getCreditTransferTransactionInformation13() {
        MxPacs00900102 obj =  (MxPacs00900102)map.get(Mx2MtConstants.MX_OBJECT);
        if (obj == null || obj.getFinInstnCdtTrf() == null) {
            return null;
        }
        FinancialInstitutionCreditTransferV02 finInstnCdtTrf = obj.getFinInstnCdtTrf();
        List<CreditTransferTransactionInformation13> cdtTrfTxInf = finInstnCdtTrf.getCdtTrfTxInf();
        if (cdtTrfTxInf.size() > 0) {
            return cdtTrfTxInf.get(0);
        } else {
            return null;
        }
    }
}
