package com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00900102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * 1，若<BICFI>有值，则转为“/INS/此节点的值”
 * 2，如果<Nm>有值，则取值为“/INS/”+此节点的值，超过35字节则将剩下内容用“//内容“的方式进行截取
 * 3，如果节点<InstrForCdtrAgt><Cd>没有值，但节点<InstrForCdtrAgt><InstrInf>有值，则取值“/ACC/”+<InstrForCdtrAgt><InstrInf>节点的值
 * 4，如果节点<InstrForCdtrAgt><Cd>值为PHOB，则取值“/PHONBEN/”+<InstrForCdtrAgt><InstrInf>节点的值
 * 5，如果节点<InstrForCdtrAgt><Cd>值为TELB，则取值“/PHONBEN/”+<InstrForCdtrAgt><InstrInf>节点的值
 * 6，如果节点<InstrForNxtAgt><Cd>值为PHOA，且56a不存在，则取值“/PHON/”+<InstrForNxtAgt><InstrInf>节点的值
 * 7，如果节点<InstrForNxtAgt><Cd>值为TELA，则取值“/TELE/”+<InstrForNxtAgt><InstrInf>节点的值
 * 8，如果节点<InstrForNxtAgt><Cd>值为空，且<InstrInf>节点有值，则取值“/REC/”+<InstrForNxtAgt><InstrInf>节点的值
 * 9，如果节点<InstrForNxtAgt><Cd>值为PHOA，且56a存在，则取值“/PHONIBK/”+<InstrForNxtAgt><InstrInf>节点的值
 * <CdtTrfTxInf>[1]<PrvsInstgAgt1><FinInstnId><BICFI>
 * <CdtTrfTxInf>[1]<PrvsInstgAgt1><FinInstnId><Nm>
 * <CdtTrfTxInf>[1]<InstrForCdtrAgt><Cd>
 * <CdtTrfTxInf>[1]<InstrForCdtrAgt><InstrInf>
 * <CdtTrfTxInf>[1]<InstrForNxtAgt><Cd>
 * <CdtTrfTxInf>[1]<InstrForNxtAgt><InstrInf>
 * <CdtTrfTxInf>[1]<RmtInf><Ustrd>
 */
public class Field72Generate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl.Field72Generate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            CreditTransferTransactionInformation13 cdtTrfTxInf = getCreditTransferTransactionInformation13();
            StringBuilder sb = new StringBuilder();
            String field72 = "";
            if (cdtTrfTxInf != null) {
                BranchAndFinancialInstitutionIdentification4 prvsInstgAgt = cdtTrfTxInf.getPrvsInstgAgt();
                if (prvsInstgAgt != null && prvsInstgAgt.getFinInstnId() != null) {
                    String bicCode = prvsInstgAgt.getFinInstnId().getBIC();
                    if (StringUtil.isEmpty(bicCode)) {
                        bicCode = prvsInstgAgt.getFinInstnId().getNm();
                    }
                    if (StringUtil.isNotEmpty(bicCode)) {
                        bicCode = Mx2MtConstants.INS + bicCode;
                        if (bicCode.length() > 35) {
                            bicCode = bicCode.substring(0, 35);
                        }
                        if (sb.length() == 0) {
                            sb.append(bicCode);
                        } else {
                            sb.append(Mx2MtConstants.NEW_LINE).append(bicCode);
                        }
                    }
                }
                List<InstructionForCreditorAgent2> instrForCdtrAgtList = cdtTrfTxInf.getInstrForCdtrAgt();
                if (instrForCdtrAgtList.size() > 0) {
                    String cdtrAgtCode = "";
                    InstructionForCreditorAgent2 instrForCdtrAgt = instrForCdtrAgtList.get(0);
                    if (instrForCdtrAgt.getCd() == null && StringUtil.isNotEmpty(instrForCdtrAgt.getInstrInf())) {
                        cdtrAgtCode = Mx2MtConstants.ACC + instrForCdtrAgt.getInstrInf();
                    } else if (Instruction5Code.PHOB.equals(instrForCdtrAgt.getCd()) && StringUtil.isNotEmpty(instrForCdtrAgt.getInstrInf())) {
                        cdtrAgtCode = Mx2MtConstants.PHONBEN + instrForCdtrAgt.getInstrInf();
                    } else if (Instruction5Code.TELB.equals(instrForCdtrAgt.getCd()) && StringUtil.isNotEmpty(instrForCdtrAgt.getInstrInf())) {
                        cdtrAgtCode = Mx2MtConstants.PHONBEN + instrForCdtrAgt.getInstrInf();
                    }
                    if (StringUtil.isNotEmpty(cdtrAgtCode)) {
                        if (cdtrAgtCode.length() > 35) {
                            cdtrAgtCode = cdtrAgtCode.substring(0, 35);
                        }
                        if (sb.length() == 0) {
                            sb.append(cdtrAgtCode);
                        } else {
                            sb.append(Mx2MtConstants.NEW_LINE).append(cdtrAgtCode);
                        }
                    }
                }
                List<InstructionForNextAgent1> instrForNxtAgtList = cdtTrfTxInf.getInstrForNxtAgt();
                if (instrForNxtAgtList.size() > 0) {
                    String nextAgentCode = "";
                    InstructionForNextAgent1 instrForNxtAgt = instrForNxtAgtList.get(0);
                    if (instrForNxtAgt.getCd() == null && StringUtil.isNotEmpty(instrForNxtAgt.getInstrInf())) {
                        nextAgentCode = Mx2MtConstants.REC + instrForNxtAgt.getInstrInf();
                    } else if (Instruction4Code.TELA.equals(instrForNxtAgt.getCd()) && StringUtil.isNotEmpty(instrForNxtAgt.getInstrInf())) {
                        nextAgentCode = Mx2MtConstants.TELE + instrForNxtAgt.getInstrInf();
                    }  else if (Instruction4Code.PHOA.equals(instrForNxtAgt.getCd()) && StringUtil.isNotEmpty(instrForNxtAgt.getInstrInf())) {
                        String field56Option = (String)map.get("field56Option");
                        if (StringUtil.isEmpty(field56Option)) {
                            nextAgentCode = Mx2MtConstants.PHON + instrForNxtAgt.getInstrInf();
                        } else {
                            nextAgentCode = Mx2MtConstants.PHONIBK + instrForNxtAgt.getInstrInf();
                        }
                    }
                    if (StringUtil.isNotEmpty(nextAgentCode)) {
                        if (nextAgentCode.length() > 35) {
                            nextAgentCode = nextAgentCode.substring(0, 35);
                        }
                        if (sb.length() == 0) {
                            sb.append(nextAgentCode);
                        } else {
                            sb.append(Mx2MtConstants.NEW_LINE).append(nextAgentCode);
                        }
                    }
                }
            }
            if (sb.length() > 0) {
                field72 = sb.toString();
            }
            map.put("field72", field72);
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT202 filed72 error");
        }

    }

    public CreditTransferTransactionInformation13 getCreditTransferTransactionInformation13() {
        MxPacs00900102 obj =  (MxPacs00900102)map.get(Mx2MtConstants.MX_OBJECT);
        if (obj == null || obj.getFinInstnCdtTrf() == null) {
            return null;
        }
        FinancialInstitutionCreditTransferV02 finInstnCdtTrf = obj.getFinInstnCdtTrf();
        List<CreditTransferTransactionInformation13> cdtTrfTxInf = finInstnCdtTrf.getCdtTrfTxInf();
        if (cdtTrfTxInf.size() > 0) {
            return cdtTrfTxInf.get(0);
        } else {
            return null;
        }
    }
}
