package com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl.cov;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00900102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 1，若<AnyBIC>节点有值，则取为B\50A，且BIC取此值
 * 2，若节点<IBAN>有值，则在此节点值前面加上“/”后，作为50A的账号；
 * 3，若节点<Othr><SchmeNm><Cd>有值，则取“//CH”+<Othr><Id>，则作为B\50A的账号；否则直接取<Othr><Id>，则作为B\50A的账号。
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Dbtr><Id><OrgId><AnyBIC>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAcct><Id><Othr><SchmeNm><Cd>
 *      * 1，待补充？？？
 *  * 2，账号取值规则与B\50A相同
 *   * 1，若<Dbtr><Nm>有录入，则设置B\50 option为“K”，且名称取自此节点，而地址需要合并截取？？？
 *  * 2，账号取值规则与B\50A相同
 *  * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Dbtr><Nm>
 *  * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Dbtr><PstlAdr><AdrLine>
 */
public class Field50AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field50AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        // TODO 50F取值待补充
        try {
            CreditTransferTransactionInformation12 undrlygCstmrCdtTrf = getUndrlygCstmrCdtTrf();
            String field50BOption = "";
            String field50B = "";
            if (undrlygCstmrCdtTrf != null) {
                PartyIdentification32 dbtr = undrlygCstmrCdtTrf.getDbtr();
                if (dbtr != null) {
                    Party6Choice id = dbtr.getId();
                    if (dbtr.getId() != null
                        && dbtr.getId().getOrgId() != null
                        && StringUtil.isNotEmpty(dbtr.getId().getOrgId().getBICOrBEI())) {
                            String bicCode = dbtr.getId().getOrgId().getBICOrBEI();
                            field50BOption = "A";
                            String account = getMtFormatAccount(undrlygCstmrCdtTrf.getDbtrAcct());
                            if (!StringUtil.isEmpty(account)) {
                                field50B = account + Mx2MtConstants.NEW_LINE + bicCode;
                            } else {
                                field50B = bicCode;
                            }
                        } else if (StringUtil.isNotEmpty(dbtr.getNm())) {
                            field50BOption = "K";
                            String nameAddr = "";
                            PostalAddress6 pstlAdr = dbtr.getPstlAdr();
                            if (pstlAdr != null) {
                                List<String> adrLineList = pstlAdr.getAdrLine();
                                if (adrLineList.size() > 0) {
                                    for (int i=0; i<adrLineList.size(); i++) {
                                        if (i == 4) {
                                            break;
                                        }
                                        String adrLine = adrLineList.get(i);
                                        if (adrLine.length() > 35) {
                                            adrLine = adrLine.substring(0, 35);
                                        }
                                        if (i == 0) {
                                            nameAddr += adrLine;
                                        } else {
                                            nameAddr += Mx2MtConstants.NEW_LINE+adrLine;
                                        }
                                    }
                                }
                            }
                            String account = getMtFormatAccount(undrlygCstmrCdtTrf.getDbtrAcct());
                            if (!StringUtil.isEmpty(account)) {
                                field50B = account + Mx2MtConstants.NEW_LINE + nameAddr;;
                            } else {
                                field50B = nameAddr;
                            }
                        }
                    }
                }
            map.put("field50BOption", field50BOption);
            map.put("field50B", field50B);
        } catch(Exception e) {
            throw new SwiftException("ERROR", "Convert MT202COV Field50A error");
        }
    }

    public CreditTransferTransactionInformation12 getUndrlygCstmrCdtTrf() {
        MxPacs00900102 obj =  (MxPacs00900102)map.get(Mx2MtConstants.MX_OBJECT);
        if (obj == null || obj.getFinInstnCdtTrf() == null) {
            return null;
        }
        FinancialInstitutionCreditTransferV02 finInstnCdtTrf = obj.getFinInstnCdtTrf();
        List<CreditTransferTransactionInformation13> cdtTrfTxInf = finInstnCdtTrf.getCdtTrfTxInf();
        if (cdtTrfTxInf.size() > 0) {
            return cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf();
        } else {
            return null;
        }
    }
}

