package com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl.cov;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00900102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 1，若<BICFI>节点有值，则取为B\52A，且BIC取此值
 * 2，若节点<IBAN>有值，则在此节点值前面加上“/”后，作为52A的账号；
 * 3，若节点<Othr><SchmeNm><Cd>有值，则取“//CH”+<Othr><Id>，则作为B\50A的账号；否则直接取<Othr><Id>，则作为B\52A的账号。
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgt><FinInstnId><BICFI>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgtAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgtAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgtAcct><Id><Othr><SchmeNm><Cd>
 *      * 1，若节点<Nm>有值，则设置B\52 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 *  * 2，账号取值规则和节点与B\52A相同
 *  * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgt><FinInstnId><Nm>
 *  * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><DbtrAgt><FinInstnId><PstlAdr><AdrLine>[1]，<AdrLine>[2]，<AdrLine>[3]
 */
public class FieldB52AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(FieldB52AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            CreditTransferTransactionInformation12 undrlygCstmrCdtTrf = getUndrlygCstmrCdtTrf();
            String field52BOption = "";
            String field52B = "";
            if (undrlygCstmrCdtTrf != null) {
                BranchAndFinancialInstitutionIdentification4 dbtr = undrlygCstmrCdtTrf.getDbtrAgt();
                if (dbtr != null) {
                    FinancialInstitutionIdentification7 finInstnId = dbtr.getFinInstnId();
                    if (finInstnId != null) {
                        if (StringUtil.isNotEmpty(finInstnId.getBIC())) {
                            field52BOption = "A";
                            String bicCode = finInstnId.getBIC();
                            String account = getMtFormatAccount(undrlygCstmrCdtTrf.getDbtrAgtAcct());
                            if (!StringUtil.isEmpty(account)) {
                                field52B = account + Mx2MtConstants.NEW_LINE + bicCode;
                            } else {
                                field52B = bicCode;
                            }
                        } else if (StringUtil.isNotEmpty(finInstnId.getNm())) {
                            field52BOption = "D";
                            String nameAddr = "";
                            PostalAddress6 pstlAdr = finInstnId.getPstlAdr();
                            if (pstlAdr != null) {
                                List<String> adrLineList = pstlAdr.getAdrLine();
                                if (adrLineList.size() > 0) {
                                    for (int i=0; i<adrLineList.size(); i++) {
                                        if (i == 4) {
                                            break;
                                        }
                                        String adrLine = adrLineList.get(i);
                                        if (adrLine.length() > 35) {
                                            adrLine = adrLine.substring(0, 35);
                                        }
                                        if (i == 0) {
                                            nameAddr += adrLine;
                                        } else {
                                            nameAddr += Mx2MtConstants.NEW_LINE+adrLine;
                                        }
                                    }
                                }
                            }
                            String account = getMtFormatAccount(undrlygCstmrCdtTrf.getDbtrAgtAcct());
                            if (!StringUtil.isEmpty(account)) {
                                field52B = account + Mx2MtConstants.NEW_LINE + nameAddr;;
                            } else {
                                field52B = nameAddr;
                            }
                        }
                    }
                }
            }
            map.put("field52BOption", field52BOption);
            map.put("field52B", field52B);
        } catch(Exception e) {
            throw new SwiftException("ERROR", "Convert MT202COV Field52 error");
        }

    }

    public CreditTransferTransactionInformation12 getUndrlygCstmrCdtTrf() {
        MxPacs00900102 obj =  (MxPacs00900102)map.get(Mx2MtConstants.MX_OBJECT);
        if (obj == null || obj.getFinInstnCdtTrf() == null) {
            return null;
        }
        FinancialInstitutionCreditTransferV02 finInstnCdtTrf = obj.getFinInstnCdtTrf();
        List<CreditTransferTransactionInformation13> cdtTrfTxInf = finInstnCdtTrf.getCdtTrfTxInf();
        if (cdtTrfTxInf.size() > 0) {
            return cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf();
        } else {
            return null;
        }
    }
}
