package com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl.cov;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00900102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 1,若节点<BICFI>有值，则设置B\56 option为“A”，同时BIC取此栏位；
 * 2,若节点<IBAN>有值，则作为56A的账号；
 * 3,若节点<Othr><SchmeNm><Cd>有值，则取“//CH”+<Othr><Id>，则作为B\56A的账号；否则直接取<Othr><Id>，则作为B\56A的账号。
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><IntrmyAgt1><FinInstnId><BICFI>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><IntrmyAgt1Acct><Id><IBAN>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><IntrmyAgt1Acct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><IntrmyAgt1Acct><Id><Othr><SchmeNm><Cd>
 * 56c
 * 如果<IntrmyAgt1><FinInstnId><BICFI>为空，且<IntrmyAgt1><FinInstnId><Nm>为空，但节点<IBAN>有值，或者节点<Othr><Id>有值，则设置B\56option为“C”
 * 若节点<Id><Othr><Id>不为空，则取“//CH”+此节点作为账号，否则取节点<MmbId>作为账号：
 <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><IntrmyAgt1><FinInstnId><ClrSysMmbId><ClrSysId><Cd>
 <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><IntrmyAgt1><FinInstnId><ClrSysMmbId><ClrSysId><Prtry>
 <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><IntrmyAgt1><FinInstnId><ClrSysMmbId><MmbId>
 * 56d
 * 1，若节点<Nm>有值，则设置B\56 option为“D”，且名称取自此节点，而地址需要合并截取？？？
 * 2，账号取值规则和节点与B\56A相同
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><IntrmyAgt1><FinInstnId><Nm>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><IntrmyAgt1><FinInstnId><PstlAdr><AdrLine>[1],<AdrLine>[2],<AdrLine>[3]
 */
public class FieldB56Generate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(FieldB56Generate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            CreditTransferTransactionInformation12 undrlygCstmrCdtTrf = getUndrlygCstmrCdtTrf();
            String field56BOption = "";
            String field56B = "";
            if (undrlygCstmrCdtTrf != null) {
                BranchAndFinancialInstitutionIdentification4 intrmyAgt1 = undrlygCstmrCdtTrf.getIntrmyAgt1();
                if (intrmyAgt1 != null) {
                    FinancialInstitutionIdentification7 finInstnId = intrmyAgt1.getFinInstnId();
                    if (finInstnId != null) {
                        if (StringUtil.isNotEmpty(finInstnId.getBIC())) {
                            field56BOption = "A";
                            String bicCode = finInstnId.getBIC();
                            String account = getMtFormatAccount(undrlygCstmrCdtTrf.getIntrmyAgt1Acct());
                            if (!StringUtil.isEmpty(account)) {
                                field56B = account + Mx2MtConstants.NEW_LINE + bicCode;
                            } else {
                                field56B = bicCode;
                            }
                        } else if (StringUtil.isNotEmpty(finInstnId.getNm())) {
                            field56BOption = "D";
                            String nameAddr = "";
                            PostalAddress6 pstlAdr = finInstnId.getPstlAdr();
                            if (pstlAdr != null) {
                                List<String> adrLineList = pstlAdr.getAdrLine();
                                if (adrLineList.size() > 0) {
                                    for (int i = 0; i < adrLineList.size(); i++) {
                                        if (i == 4) {
                                            break;
                                        }
                                        String adrLine = adrLineList.get(i);
                                        if (adrLine.length() > 35) {
                                            adrLine = adrLine.substring(0, 35);
                                        }
                                        if (i == 0) {
                                            nameAddr += adrLine;
                                        } else {
                                            nameAddr += Mx2MtConstants.NEW_LINE + adrLine;
                                        }
                                    }
                                }
                            }
                            String account = getMtFormatAccount(undrlygCstmrCdtTrf.getIntrmyAgt1Acct());
                            if (!StringUtil.isEmpty(account)) {
                                field56B = account + Mx2MtConstants.NEW_LINE + nameAddr;
                            } else {
                                field56B = nameAddr;
                            }
                        } else {
                            if (finInstnId.getClrSysMmbId() != null
                                    && StringUtil.isNotEmpty(finInstnId.getClrSysMmbId().getMmbId())) {
                                field56BOption = "C";
                                field56B = "/" + finInstnId.getClrSysMmbId().getMmbId();
                            }
                        }
                    }
                }
            }
            map.put("field56BOption", field56BOption);
            map.put("field56B", field56B);
        } catch(Exception e) {
            throw new SwiftException("ERROR", "Convert MT202COV Field56 error");
        }
    }

    public CreditTransferTransactionInformation12 getUndrlygCstmrCdtTrf() {
        MxPacs00900102 obj =  (MxPacs00900102)map.get(Mx2MtConstants.MX_OBJECT);
        if (obj == null || obj.getFinInstnCdtTrf() == null) {
            return null;
        }
        FinancialInstitutionCreditTransferV02 finInstnCdtTrf = obj.getFinInstnCdtTrf();
        List<CreditTransferTransactionInformation13> cdtTrfTxInf = finInstnCdtTrf.getCdtTrfTxInf();
        if (cdtTrfTxInf.size() > 0) {
            return cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf();
        } else {
            return null;
        }
    }
}
