package com.prowidesoftware.brilliance.mx2mt.mt202cov202.impl.cov;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxPacs00900102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 1,若节点<Nm>有值，则设置B\59 option为“”，且名称取自此节点，而地址需要合并截取？？？
 * 2,若节点<IBAN>有值，则在此节点的值前面加上“/”后，作为B\59账号的值；
 * 3,若节点<Othr><Id>有值，且节点<Othr><SchmeNm><Cd>有值，则在节点<Othr><Id>的值前面加上“//CH”后，作为B\59账号的值；
 * 4,若节点<Othr><Id>有值，且节点<Othr><SchmeNm><Cd>没值，则在节点<Othr><Id>的值前面加上“/”后，作为B\59账号的值；
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Cdtr><Nm>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Cdtr><PstlAdr><AdrLine>[1]，<AdrLine>[2]，<AdrLine>[3]
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><CdtrAcct><Id><IBAN>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><CdtrAcct><Id><Othr><Id>
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><CdtrAcct><Id><Othr><SchmeNm><Cd>
 * 1,若节点<AnyBIC>有值，则设置B\59 option为“A”，同时BIC取此栏位；
 * 2，账号取值规则和节点与B\59相同
 * <CdtTrfTxInf>[1]<UndrlygCstmrCdtTrf><Cdtr><Id><OrgId><AnyBIC>
 */
public class FieldB59Generate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(FieldB59Generate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        try {
            CreditTransferTransactionInformation12 undrlygCstmrCdtTrf = getUndrlygCstmrCdtTrf();
            String field59BOption = "";
            String field59B = "";
            if (undrlygCstmrCdtTrf != null) {
                PartyIdentification32 cdtr = undrlygCstmrCdtTrf.getCdtr();
                if (cdtr != null) {
                    if (StringUtil.isNotEmpty(cdtr.getNm())) {
                        field59BOption = "NOLETTER";
                        String nameAddr = "";
                        PostalAddress6 pstlAdr = cdtr.getPstlAdr();
                        if (pstlAdr != null) {
                            List<String> adrLineList = pstlAdr.getAdrLine();
                            if (adrLineList.size() > 0) {
                                for (int i=0; i<adrLineList.size(); i++) {
                                    if (i == 4) {
                                        break;
                                    }
                                    String adrLine = adrLineList.get(i);
                                    if (adrLine.length() > 35) {
                                        adrLine = adrLine.substring(0, 35);
                                    }
                                    if (i == 0) {
                                        nameAddr += adrLine;
                                    } else {
                                        nameAddr += Mx2MtConstants.NEW_LINE+adrLine;
                                    }
                                }
                            }
                        }
                        String account = getMtFormatAccount(undrlygCstmrCdtTrf.getCdtrAcct());
                        if (!StringUtil.isEmpty(account)) {
                            field59B = account + Mx2MtConstants.NEW_LINE + nameAddr;;
                        } else {
                            field59B = nameAddr;
                        }
                    } else if (cdtr.getId() != null
                            && cdtr.getId().getOrgId() != null
                            && StringUtil.isNotEmpty(cdtr.getId().getOrgId().getBICOrBEI())) {
                        String bicCode = cdtr.getId().getOrgId().getBICOrBEI();
                        field59BOption = "A";
                        String account = getMtFormatAccount(undrlygCstmrCdtTrf.getCdtrAcct());
                        if (!StringUtil.isEmpty(account)) {
                            field59B = account + Mx2MtConstants.NEW_LINE + bicCode;
                        } else {
                            field59B = bicCode;
                        }
                    }
                }
            }
            map.put("field59BOption", field59BOption);
            map.put("field59B", field59B);
        } catch(Exception e) {
            throw new SwiftException("ERROR", "Convert MT202COV Field59 error");
        }
    }

    public CreditTransferTransactionInformation12 getUndrlygCstmrCdtTrf() {
        MxPacs00900102 obj =  (MxPacs00900102)map.get(Mx2MtConstants.MX_OBJECT);
        if (obj == null || obj.getFinInstnCdtTrf() == null) {
            return null;
        }
        FinancialInstitutionCreditTransferV02 finInstnCdtTrf = obj.getFinInstnCdtTrf();
        List<CreditTransferTransactionInformation13> cdtTrfTxInf = finInstnCdtTrf.getCdtTrfTxInf();
        if (cdtTrfTxInf.size() > 0) {
            return cdtTrfTxInf.get(0).getUndrlygCstmrCdtTrf();
        } else {
            return null;
        }
    }
}


